def get_link(name: str, number: int):
    """ Give the link to example picture from library repository. Choose the group: cats, vegetables or butterflies. Then choose the number of picture."""
    
    return f'https://raw.githubusercontent.com/macarell228/graphical_password/main/examples/{name}/{number}.avif'